<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (info@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Domain\Repository;

use IMIA\ImiaBaseExt\Domain\Repository\NoStorageRepository;
use IMIA\ImiaBaseExt\Utility\Helper;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Object\ObjectManager;
use IMIA\ImiaPageteaser\Domain\Model\Category;
use IMIA\ImiaPageteaser\Domain\Model\Page;
use IMIA\ImiaPageteaser\Domain\Model\PageAlt;

/**
 * @package     imia_pageteaser
 * @subpackage  Domain\Repository
 * @author      David Frerich <d.frerich@imia.de>
 */
class PageRepository extends NoStorageRepository
{
    /**
     * @var PageAltRepository
     */
    public $pageAltRepository;

    /**
     * @var CategoryRepository
     */
    public $categoryRepository;

    /**
     * @var boolean
     */
    protected $showAll = false;

    /**
     * @param bool $ignoreEnableFields
     * @return $this
     */
    public function setIgnoreEnableFields($ignoreEnableFields)
    {
        $this->getPageAltRepository()->setIgnoreEnableFields($ignoreEnableFields);

        return parent::setIgnoreEnableFields($ignoreEnableFields);
    }

    /**
     * @param boolean $showAll
     * @return $this
     */
    public function setShowAll($showAll)
    {
        $this->showAll = $showAll;

        return $this;
    }

    /**
     * @param array $uids
     * @param string $date
     * @param array $excludeUids
     * @return integer
     */
    public function countByUids(array $uids, $date = 'all', $excludeUids = array())
    {
        if (count($uids) > 0) {
            $query = $this->createQueryByUids($uids, $date, $excludeUids);

            return $query->count();
        } else {
            return 0;
        }
    }

    /**
     * @param array $uids
     * @param string $date
     * @param array $excludeUids
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByUids(array $uids, $date = 'all', $excludeUids = array(), $limit = null, $offset = null, $sorting = null)
    {
        if (count($uids) > 0) {
            $query = $this->createQueryByUids($uids, $date, $excludeUids, $limit, $offset, $sorting);

            return $this->executeQuery($query, $limit, $offset, $sorting);
        } else {
            return array();
        }
    }

    /**
     * @param array $uids
     * @return array
     */
    public function findTeaserPagesByUids(array $uids)
    {
        if (count($uids) > 0) {
            $query = $this->createQuery();
            $query->matching(
                $query->in('uid', $uids)
            );

            $uids = array();
            $result = $this->executeArrayQuery($query, array('teaser_pages'));
            foreach ($result as $row) {
                $uids = array_merge($uids, array_map('intval', explode(',', $row['teaser_pages'])));
            }

            return array_unique($uids);
        } else {
            return array();
        }
    }

    /**
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param array $sorting
     * @return array
     */
    public function findUidsByDoktypesAndCategories(array $doktypes, $categories = null,
        array $filter = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByDoktypesAndCategories($doktypes, $categories, $filter, null, null, $sorting);

        $pageUids = $this->executeUidArrayQuery($query);
        $this->applyFilterForLanguage($pageUids, $filter);

        return $pageUids;
    }

    /**
     * @param array $pIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param array $sorting
     * @return array
     */
    public function findUidsByPidsAndDoktypesAndCategories(array $pIds, array $doktypes, $categories = null,
        array $filter = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByPidsAndDoktypesAndCategories(
            $this->getResolvedPageUids($pIds), $doktypes, $categories, $filter, null, null, $sorting);

        $pageUids = $this->executeUidArrayQuery($query);
        $this->applyFilterForLanguage($pageUids, $filter);

        return $pageUids;
    }

    /**
     * @param int $parentPageId
     * @param int $archiveDateSeconds
     * @return array
     */
    public function findByPidAndArchivedateOlderThan($parentPageId, $archiveDateSeconds)
    {
        $query = $this->createQuery();

        $constraints = array(
            $query->equals('pid',$parentPageId),
            $query->greaterThan('archivedate', 0),
            $query->lessThanOrEqual('archivedate', (time() - $archiveDateSeconds))
        );

        $query->matching($query->logicalAnd($constraints));

        return $this->executeQuery($query);
    }

    /**
     * @param array $uIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param array $sorting
     * @return array
     */
    public function findUidsByUidsAndDoktypesAndCategories(array $uIds, array $doktypes, $categories = null,
        array $filter = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByUidsAndDoktypesAndCategories($uIds, $doktypes, $categories, $filter, null, null, $sorting);

        $pageUids = $this->executeUidArrayQuery($query);
        $this->applyFilterForLanguage($pageUids, $filter);

        return $pageUids;
    }

    /**
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @return integer
     */
    public function countByDoktypesAndCategories(array $doktypes, $categories = null, array $filter = null)
    {
        $query = $this->createQueryByDoktypesAndCategories($doktypes, $categories, $filter);

        return $query->count();
    }

    /**
     * @param array $pIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @return integer
     */
    public function countByPidsAndDoktypesAndCategories(array $pIds, array $doktypes, $categories = null, array $filter = null)
    {
        $query = $this->createQueryByPidsAndDoktypesAndCategories(
            $this->getResolvedPageUids($pIds), $doktypes, $categories, $filter);

        return $query->count();
    }

    /**
     * @param array $uIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @return integer
     */
    public function countByUidsAndDoktypesAndCategories(array $uIds, array $doktypes, $categories = null, array $filter = null)
    {
        $query = $this->createQueryByUidsAndDoktypesAndCategories($uIds, $doktypes, $categories, $filter);

        return $query->count();
    }

    /**
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return array
     */
    public function findByDoktypesAndCategories(array $doktypes, $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByDoktypesAndCategories($doktypes, $categories, $filter, $limit, $offset, $sorting);

        if ($limit == 1) { /** @FIXME Problem with limit and pages not available/hidden in language */
            return array($query->execute()->getFirst());
        } else {
            return $this->executeQuery($query, $limit, $offset, $sorting);
        }
    }

    /**
     * @param array $pIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return array
     */
    public function findByPidsAndDoktypesAndCategories(array $pIds, array $doktypes, $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByPidsAndDoktypesAndCategories(
            $this->getResolvedPageUids($pIds), $doktypes, $categories, $filter, $limit, $offset, $sorting);

        if ($limit == 1) { /** @FIXME Problem with limit and pages not available/hidden in language */
            return array($query->execute()->getFirst());
        } else {
            return $this->executeQuery($query, $limit, $offset, $sorting);
        }
    }

    /**
     * @param array $uIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return array
     */
    public function findByUidsAndDoktypesAndCategories(array $uIds, array $doktypes, $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByUidsAndDoktypesAndCategories($uIds, $doktypes, $categories, $filter, $limit, $offset, $sorting);

        if ($limit == 1) { /** @FIXME Problem with limit and pages not available/hidden in language */
            return array($query->execute()->getFirst());
        } else {
            return $this->executeQuery($query, $limit, $offset, $sorting);
        }
    }

    /**
     * @param array $pIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryInterface
     */
    public function createQueryByPidsAndDoktypesAndCategories(array $pIds, array $doktypes, $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        if (count($pIds) == 0) {
            $pIds = array(0);
        }

        $query = $this->createQuery($limit, $offset, $sorting);
        $constraints = array(
            $query->in('pid', $pIds),
            $query->in('doktype', $doktypes)
        );
        $this->addCategoriesConstraints($query, $constraints, $categories);
        $this->addFilterConstraints($query, $constraints, $filter);

        $query->matching(
            $query->logicalAnd($constraints)
        );

        return $query;
    }

    /**
     * @param array $uIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryInterface
     */
    public function createQueryByUidsAndDoktypesAndCategories(array $uIds, array $doktypes, $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQuery($limit, $offset, $sorting);
        $constraints = array(
            $query->in('uid', $uIds),
            $query->in('doktype', $doktypes)
        );
        $this->addCategoriesConstraints($query, $constraints, $categories);
        $this->addFilterConstraints($query, $constraints, $filter);

        $query->matching(
            $query->logicalAnd($constraints)
        );

        return $query;
    }

    /**
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryInterface
     */
    public function createQuery($limit = null, $offset = null, $sorting = null)
    {
        // fetch all pages, cause we can't sort now
        if ($this->getLanguageUid() > 0 && $sorting && !in_array($sorting[0], array('sorting', 'crdate', 'tstamp', 'lastUpdated'))) {
            $limit = null;
            $offset = null;
        }

        return parent::createQuery($limit, $offset, $sorting);
    }

    /**
     * @param array $uids
     * @param string $date
     * @param array $excludeUids
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryInterface
     */
    protected function createQueryByUids(array $uids, $date = 'all', $excludeUids = array(), $limit = null, $offset = null, $sorting = null)
    {
        $query = $this->createQuery($limit, $offset, $sorting);

        $constraints = array($query->in('uid', $uids));
        if ($excludeUids) {
            $constraints[] = $query->logicalNot($query->in('uid', $excludeUids));
        }

        switch ($date) {
            case 'future':
                $constraints[] = $query->greaterThanOrEqual('lastUpdated', time());
                break;
            case 'past':
                $constraints[] = $query->lessThanOrEqual('lastUpdated', time());
                break;
            default:
                break;
        }

        $query->matching($query->logicalAnd($constraints));

        return $query;
    }

    /**
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryInterface
     */
    protected function createQueryByDoktypesAndCategories(array $doktypes, $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQuery($limit, $offset, $sorting);

        $constraints = array(
            $query->in('doktype', $doktypes)
        );
        $this->addCategoriesConstraints($query, $constraints, $categories);
        $this->addFilterConstraints($query, $constraints, $filter);

        $query->matching(
            $query->logicalAnd($constraints)
        );

        return $query;
    }

    /**
     * @param array $pageUids
     * @return array
     */
    public function getResolvedPageUids(array $pageUids)
    {
        $query = $this->createQuery();
        $query->matching($query->in('uid', $pageUids));

        $resolvedPageUids = array();
        foreach ($query->execute() as $page) {
            if ($page->getDoktype() == 7 && $page->getMountPid()) {
                $resolvedPageUids[$page->getUid()] = $page->getMountPid();
            } elseif ($page->getContentFromPid()) {
                $resolvedPageUids[$page->getUid()] = $page->getContentFromPid();
            } else {
                $resolvedPageUids[$page->getUid()] = $page->getUid();
            }
        }

        return $resolvedPageUids;
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
     * @param array $constraints
     * @param array $categories
     */
    protected function addCategoriesConstraints($query, array &$constraints, $categories = null)
    {
        if ($categories && count($categories) > 0) {
            $categoryConstraints = array();
            foreach ($categories as $category) {
                /** @var Category $category */
                $categoryConstraints[] = $query->contains('categories', $category);
            }
            $constraints[] = $query->logicalOr($categoryConstraints);
        }
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
     * @param array $constraints
     * @param array $filter
     */
    protected function addFilterConstraints($query, array &$constraints, array $filter = null)
    {
        if ($filter && count($filter) > 0) {
            if ($this->getLanguageUid() == 0) {
                if (array_key_exists('word', $filter) && $filter['word']) {
                    $searchWordContraints = array();
                    foreach ($this->getSearchWordFields() as $field) {
                        $searchWordContraints[] = $query->like($field, '%' . $filter['word'] . '%');
                    }
                    if (count($searchWordContraints) > 0) {
                        $constraints[] = $query->logicalOr($searchWordContraints);
                    }
                }

                if (array_key_exists('abc', $filter) && $filter['abc']) {
                    $abc = explode('|', $filter['abc']);

                    $abcConstraints = array();
                    foreach ($abc as $letter) {
                        $letter = strtolower(substr(trim($letter), 0, 1));
                        if ($letter) {
                            $abcConstraints[] = $query->like('title', $letter . '%');
                        }
                    }

                    if (count($abcConstraints) > 0) {
                        $constraints[] = $query->logicalOr($abcConstraints);
                    }
                }
            }

            if (array_key_exists('year', $filter) && $filter['year']) {
                if (array_key_exists('month', $filter) && $filter['month']) {
                    if (array_key_exists('day', $filter) && $filter['day']) {
                        // year and month and day
                        $constraints[] = $query->logicalAnd(array(
                            $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, (int)$filter['month'], (int)$filter['day'], (int)$filter['year'])),
                            $query->lessThan('lastUpdated', mktime(0, 0, 0, (int)$filter['month'], (int)$filter['day']+1, (int)$filter['year']))
                        ));
                    } else {
                        // year and month
                        $constraints[] = $query->logicalAnd(array(
                            $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, (int)$filter['month'], 1, (int)$filter['year'])),
                            $query->lessThan('lastUpdated', mktime(0, 0, 0, (int)$filter['month']+1, 1, (int)$filter['year']))
                        ));
                    }
                } else {
                    // years
                    $constraints[] = $query->logicalAnd(array(
                        $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, 1, 1, (int)$filter['year'])),
                        $query->lessThan('lastUpdated', mktime(0, 0, 0, 13, 1, (int)$filter['year']))
                    ));
                }
            } elseif (array_key_exists('month', $filter) && $filter['month']) {
                $monthConstraints = array();
                for ($i = 2000; $i <= (int)date('Y'); $i++) {
                    // month
                    $monthConstraints = $query->logicalAnd(array(
                        $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, (int)$filter['month'], 1, $i)),
                        $query->lessThan('lastUpdated', mktime(0, 0, 0, (int)$filter['month']+1, 1, $i))
                    ));
                }
                $constraints[] = $query->logicalOr($monthConstraints);
            } elseif (array_key_exists('month-year', $filter) && $filter['month-year']) {
                // month-year in one field
                $myFilter = explode('-', $filter['month-year']);
                if (count($myFilter) == 2) {
                    $constraints[] = $query->logicalAnd(array(
                        $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, (int)$myFilter[0], 1, (int)$myFilter[1])),
                        $query->lessThan('lastUpdated', mktime(0, 0, 0, (int)$myFilter[0]+1, 1, (int)$myFilter[1]))
                    ));
                }
            }

            if (array_key_exists('pid', $filter) && $filter['pid']) {
                $constraints[] = $query->equals('pid', (int)$filter['pid']);
            }

            if (array_key_exists('categories', $filter) && $filter['categories'] && is_array($filter['categories'])) {
                $categories = $this->getCategoryRepository()->findByUids($filter['categories']);
                $categoryContraints = array();
                foreach ($categories as $category) {
                    $categoryContraints[] = $query->contains('categories', $category);
                }

                if (count($categoryContraints) > 0) {
                    $constraints[] = $query->logicalOr($categoryContraints);
                }
            }

            if (array_key_exists('category', $filter) && $filter['category']) {
                $category = $this->getCategoryRepository()->findOneByUid((int)$filter['category']);

                if ($category) {
                    $constraints[] = $query->contains('categories', $category);
                }
            }

            $additionalFilterOptions = array_diff(array_keys($filter), $this->getFilterKeyOptions());
            foreach ($additionalFilterOptions as $filterKey) {
                if (array_key_exists($filterKey, $filter) && $filter[$filterKey]) {
                    if (is_array($filter[$filterKey])) {
                        $hasObjects = false;
                        foreach ($filter[$filterKey] as $value) {
                            if (is_object($value)) {
                                $hasObjects = true;
                            }
                        }
                        if ($hasObjects) {
                            $subconstraints = array();
                            foreach ($filter[$filterKey] as $value) {
                                if (is_object($value)) {
                                    $subconstraints[] = $query->contains($filterKey, $value);
                                } else {
                                    $subconstraints[] = $query->like($filterKey, $value);
                                }
                            }

                            $constraints[] = $query->logicalOr($subconstraints);
                        } else {
                            $constraints[] = $query->in($filterKey, $filter[$filterKey]);
                        }
                    } elseif (is_object($filter[$filterKey])) {
                        $constraints[] = $query->contains($filterKey, $filter[$filterKey]);
                    } else {
                        $constraints[] = $query->like($filterKey, $filter[$filterKey]);
                    }
                }
            }
        }
    }

    /**
     * @return array
     */
    protected function getFilterKeyOptions()
    {
        return array('word', 'abc', 'year', 'month', 'day', 'pid', 'month-year', 'categories', 'category');
    }

    /**
     * @return array
     */
    protected function getSearchWordFields()
    {
        return array('title', 'subtitle', 'abstract');
    }

    /**
     * @param array $pageUids
     * @param array $filter
     */
    protected function applyFilterForLanguage(&$pageUids, $filter)
    {
        // get all localized pages or original record
        // workaround for l18nCfg and limit
        $pages = $this->findByUids($pageUids);
        $pageUids = array();
        foreach ($pages as $page) {
            $pageUids[] = $page->getUid();
        }

        if ($this->getLanguageUid() > 0) {
            if ($filter && count($filter) > 0) {
                foreach($pages as $page) {
                    if (array_key_exists('word', $filter) && $filter['word']) {
                        if (!(strpos($page->getTitle(), $filter['word']) || strpos($page->getSubtitle(), $filter['word']) || strpos($page->getAbstract(), $filter['word']))) {
                            $pageUids = array_diff($pageUids, array($page->getPage()->getUid()));
                        }
                    }

                    if (array_key_exists('abc', $filter) && $filter['abc']) {
                        $pageLetter = strtolower(substr(trim($page->getTitle()), 0, 1));
                        $isInFilter = -1;

                        $abc = explode('|', $filter['abc']);
                        foreach ($abc as $letter) {
                            $letter = strtolower(substr(trim($letter), 0, 1));
                            if ($letter) {
                                if ($isInFilter == -1) {
                                    $isInFilter = 0;
                                }

                                if ($pageLetter == $letter) {
                                    $isInFilter = 1;
                                }
                            }
                        }

                        if ($isInFilter == 0) {
                            $pageUids = array_diff($pageUids, array($page->getPage()->getUid()));
                        }
                    }
                }
            }
        }
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return array
     */
    protected function executeQuery($query, $limit = null, $offset = null, $sorting = null)
    {
        $pages = $query->execute();

        $pageUids = array();
        $pagesArray = array();
        foreach ($pages as $page) {
            $pageUids[] = $page->getUid();
            $pagesArray[$page->getUid()] = $page;
        }

        if ($this->getLanguageUid() > 0 && count($pageUids) > 0) {
            $altPages = $this->getPageAltRepository()->findByPagesAndLanguage($pageUids, $this->getLanguageUid());
            foreach($altPages as $altPage) {
                /** @var PageAlt $altPage */
                $pagesArray[$altPage->getPage()->getUid()] = $altPage;
            }
        }


        if (!$this->showAll) {
            $newPagesArray = array();

            // l18n_cfg setting
            foreach ($pagesArray as $key => $page) {
                /** @var Page $page */
                switch ($page->getL18nCfg()) {
                    case 1:
                        // hide default translation
                        if ($this->getLanguageUid() != 0) {
                            $newPagesArray[] = $page;
                        }
                        break;
                    case 2:
                        // hide page if no translation for current language exists
                        if ($this->getLanguageUid() == $page->getSysLanguageUid()) {
                            $newPagesArray[] = $page;
                        }
                        break;
                    case 3:
                        // hide default translation
                        // hide page if no translation for current language exists
                        if ($this->getLanguageUid() != 0 && $this->getLanguageUid() == $page->getSysLanguageUid()) {
                            $newPagesArray[] = $page;
                        }
                        break;
                    case 0:
                    default:
                        $newPagesArray[] = $page;
                        break;
                }
            }

            $pagesArray = $newPagesArray;
        }

        if ($this->getLanguageUid() > 0 && $sorting && !in_array($sorting[0], array('sorting', 'crdate', 'tstamp', 'lastUpdated'))) {
            $pagesArray = Helper::sortObjectArrayByProperty($pagesArray, $sorting[0], strtolower($sorting[1]));
            $pagesArray = array_slice($pagesArray, $offset ?: 0, $limit ?: 99999999);
        }

        return $pagesArray;
    }


    /**
     * @return integer
     */
    public function getLanguageUid()
    {
        if ($this->languageUid !== null) {
            $languageUid = $this->languageUid;
        } else {
            $languageUid = 0;
            if ($GLOBALS['TSFE']) {
                $languageUid = $GLOBALS['TSFE']->sys_language_uid;
            }
        }

        return $languageUid;
    }

    /**
     * @return PageAltRepository
     */
    protected function getPageAltRepository()
    {
        if (!$this->pageAltRepository) {
            $this->pageAltRepository = $this->getObjectManager()->get(PageAltRepository::class);
        }

        return $this->pageAltRepository;
    }

    /**
     * @return CategoryRepository
     */
    protected function getCategoryRepository()
    {
        if (!$this->categoryRepository) {
            $this->categoryRepository = $this->getObjectManager()->get(CategoryRepository::class);
        }

        return $this->categoryRepository;
    }

    /**
     * @return ObjectManager
     */
    protected function getObjectManager()
    {
        return GeneralUtility::makeInstance(ObjectManager::class);
    }
}