<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2015 IMIA net based solutions (info@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Hook\Core;

use TYPO3\CMS\Backend\Routing\UriBuilder;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Core\DataHandling\DataHandler;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\HttpUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Extbase\Object\ObjectManager;

/**
 * @package     imia_pageteaser
 * @subpackage  Hook
 * @author      David Frerich <d.frerich@imia.de>
 */
class ProcessCmdmap
{
    /**
     * @param DataHandler $dataHandler
     */
    public function processCmdmap_afterFinish(DataHandler &$dataHandler)
    {
        foreach ($dataHandler->datamap as $table => $data) {
            foreach ($data as $rawId => $fields) {
                $fields['uid'] = $dataHandler->substNEWwithIDs[$rawId] ?: $rawId;

                switch ($table) {
                    case 'pages_language_overlay':
                        if (!$fields['pid']) {
                            $record = BackendUtility::getRecordWSOL($table, $fields['uid']);
                            $fields['pid'] = $record['pid'];
                        }
                        foreach ($GLOBALS['TCA']['pages_language_overlay']['columns'] as $key => $column) {
                            $fieldConfig = $column['config'];
                            if ($fieldConfig['foreign_table'] == 'sys_file_reference') {
                                $references = $this->getDb()->exec_SELECTgetRows('*', 'sys_file_reference',
                                    'fieldname = ' . $this->getDb()->fullQuoteStr($key, 'sys_file_reference') .
                                    ' AND tablenames = ' . $this->getDb()->fullQuoteStr($table, 'sys_file_reference') .
                                    ' AND pid != ' . (int)$fields['pid'] . ' AND uid_foreign = ' . (int)$fields['uid']);

                                if ($references) {
                                    foreach ($references as $fileReference) {
                                        $fileReference['pid'] = (int)$fields['pid'];

                                        $this->getDb()->exec_UPDATEquery(
                                            'sys_file_reference',
                                            'uid = ' . $fileReference['uid'],
                                            $fileReference);
                                    }
                                }
                            }
                        }
                        break;
                    case 'pages':
                        foreach ($GLOBALS['TCA']['pages']['columns'] as $key => $column) {
                            $fieldConfig = $column['config'];
                            if ($fieldConfig['foreign_table'] == 'sys_file_reference') {
                                $references = $this->getDb()->exec_SELECTgetRows('*', 'sys_file_reference',
                                    'fieldname = ' . $this->getDb()->fullQuoteStr($key, 'sys_file_reference') .
                                    ' AND tablenames = ' . $this->getDb()->fullQuoteStr($table, 'sys_file_reference') .
                                    ' AND pid != ' . (int)$fields['uid'] . ' AND uid_foreign = ' . (int)$fields['uid']);

                                if ($references) {
                                    foreach ($references as $fileReference) {
                                        $fileReference['pid'] = (int)$fields['uid'];

                                        $this->getDb()->exec_UPDATEquery(
                                            'sys_file_reference',
                                            'uid = ' . $fileReference['uid'],
                                            $fileReference);
                                    }
                                }
                            }
                        }

                        $record = BackendUtility::getRecordWSOL($table, $fields['uid']);
                        if ($record && $record['create_language']) {
                            if ($_REQUEST['_saveandclosedok'] || $_REQUEST['_savedok'] || $_REQUEST['_savedokview']) {
                                $l18nCfg = 1;
                                if (in_array((int)$fields['l18n_cfg'], array(2, 3))) {
                                    $l18nCfg = 3;
                                }

                                $this->getDb()->exec_UPDATEquery($table, 'uid = ' . $fields['uid'], array(
                                    'create_language' => 0,
                                    'l18n_cfg'        => $l18nCfg,
                                ));

                                $fileReferences = array();
                                $overlayFields = $record;
                                foreach ($overlayFields as $key => $value) {
                                    if (!array_key_exists($key, $GLOBALS['TCA']['pages_language_overlay']['columns'])) {
                                        unset($overlayFields[$key]);
                                    } else {
                                        $fieldConfig = $GLOBALS['TCA']['pages_language_overlay']['columns'][$key]['config'];

                                        if ($fieldConfig['foreign_table'] == 'sys_file_reference') {
                                            $references = $this->getDb()->exec_SELECTgetRows('*', 'sys_file_reference',
                                                'fieldname = ' . $this->getDb()->fullQuoteStr($key, 'sys_file_reference') .
                                                ' AND tablenames = ' . $this->getDb()->fullQuoteStr($table, 'sys_file_reference') .
                                                ' AND uid_foreign = ' . (int)$fields['uid']);

                                            if ($references) {
                                                foreach ($references as $fileReference) {
                                                    unset($fileReference['uid']);
                                                    $fileReferences[] = $fileReference;
                                                }
                                            }
                                        }
                                    }
                                }

                                unset($overlayFields['t3ver_label']);
                                unset($overlayFields['tx_impexp_origuid']);

                                $overlayFields['pid'] = $fields['uid'];
                                $overlayFields['sys_language_uid'] = $record['create_language'];

                                $this->getDb()->exec_INSERTquery('pages_language_overlay', $overlayFields);
                                $overlayUid = $this->getDb()->sql_insert_id();

                                foreach ($fileReferences as $fileReference) {
                                    $fileReference['uid_foreign'] = $overlayUid;
                                    $fileReference['tablenames'] = 'pages_language_overlay';

                                    $this->getDb()->exec_INSERTquery('sys_file_reference', $fileReference);
                                }

                                $returnUrl = GeneralUtility::_GP('returnUrl');

                                $objectManager = GeneralUtility::makeInstance(ObjectManager::class);
                                $uriBuilder = $objectManager->get(UriBuilder::class);
                                /** @var UriBuilder $uriBuilder */

                                if ($_REQUEST['_savedok'] || !$returnUrl) {
                                    $columnsOnly = GeneralUtility::_GP('columnsOnly');
                                    $columnsOnlyAlt = GeneralUtility::_GP('columnsOnlyAlt');
                                    $columnsOnlyTS = GeneralUtility::_GP('columnsOnlyTS');

                                    $parameters = array(
                                        'edit' => array(
                                            'pages_language_overlay' => array(
                                                $overlayUid => 'edit'
                                            ),
                                        )
                                    );
                                    if ($columnsOnly) {
                                        $parameters['edit']['pages_language_overlay']['columnsOnly'] = $columnsOnly;
                                    }
                                    if ($columnsOnlyAlt) {
                                        $parameters['edit']['pages_language_overlay']['columnsOnlyAlt'] = $columnsOnlyAlt;
                                    }
                                    if ($columnsOnlyTS) {
                                        $parameters['edit']['pages_language_overlay']['columnsOnlyTS'] = $columnsOnlyTS;
                                    }
                                    if ($returnUrl) {
                                        $parameters['returnUrl'] = $returnUrl;
                                    }

                                    $uri = $uriBuilder->buildUriFromRoute('record_edit', $parameters);
                                } elseif ($_REQUEST['_saveandclosedok']) {
                                    $uri = $uriBuilder->buildUriFromModule('web_layout', array('id' => $fields['uid'], 'language' => $fields['create_language']));
                                } else {
                                    $uri = $returnUrl;
                                }

                                HttpUtility::redirect((string)$uri);
                            }
                        }
                        break;
                }
            }
        }
    }

    /**
     * @return DatabaseConnection
     */
    protected function getDb()
    {
        return $GLOBALS['TYPO3_DB'];
    }
}