<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (info@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\ViewHelpers\Link;

use TYPO3\CMS\Extbase\DomainObject\AbstractEntity;

/**
 * @package     imia_pageteaser
 * @subpackage  ViewHelpers
 * @author      David Frerich <d.frerich@imia.de>
 */
class AdditionalParamsViewHelper extends \TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper
{
    /**
     * @var array
     */
    private static $rootLineIds = array();

    /**
     * @param object $page
     * @param object $indexPage
     * @return array
     */
    public function render(AbstractEntity $indexPage, AbstractEntity $page = null)
    {
        $additionalParams = array();
        if (!$page) {
            $page = $this->renderChildren();
        }

        if ($page) {
            if ($indexPage->getUid() == $page->getPid()) {
                $mountPage = $indexPage;
            } else {
                $mountPage = $this->objectManager
                    ->get('IMIA\ImiaPageteaser\Domain\Repository\PageRepository')
                    ->findOneByUid($page->getPid());
            }

            if ($mountPage && $mountPage->getDoktype() == 7) {
                // mount page found
            } else {
                $rootLineIds = $this->getRootlineIds();
                if (!in_array($mountPage->getPid(), $rootLineIds) && !in_array($indexPage->getUid(), $this->getRootlineIds($page->getPid()))) {
                    if (count($rootLineIds) == 1) {
                        $parentUid = $rootLineIds[0];
                    } else {
                        $parentUid = $rootLineIds[1];
                    }
                    $mountPointUid = $this->findMountPoint($parentUid, $this->getRootlineIds($page->getPid()));

                    $mountPage = $this->objectManager
                        ->get('IMIA\ImiaPageteaser\Domain\Repository\PageRepository')
                        ->findOneByUid($mountPointUid);
                } else {
                    $mountPage = null;
                }
            }

            if ($mountPage) {
                $additionalParams['MP'] = $mountPage->getMountPid() . '-' . $mountPage->getUid();
            }
        }

        return $additionalParams;
    }

    /**
     * @param integer $uid
     * @param array $mointPointUids
     * @return integer|bool
     */
    protected function findMountPoint($uid, $mointPointUids)
    {
        $result = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
            'uid, mount_pid, doktype',
            'pages',
            'pid = ' . (int)$uid . $GLOBALS['TSFE']->sys_page->enableFields('pages')
        );

        while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
            if ($row['doktype'] == 7 && in_array($row['mount_pid'], $mointPointUids)) {
                return $row['uid'];
            }

            $subresult = $this->findMountPoint($row['uid'], $mointPointUids);
            if ($subresult) {
                return $subresult;
            }
        }

        return false;
    }

    /**
     * @return array
     */
    protected function getRootlineIds($pageId = 0)
    {
        if (!isset(self::$rootLineIds[$pageId])) {
            self::$rootLineIds[$pageId] = array();

            if ($pageId) {
                if ($GLOBALS['TSFE']->sys_page) {
                    foreach ($GLOBALS['TSFE']->sys_page->getRootLine($pageId, '', true) as $rootLinePage) {
                        self::$rootLineIds[$pageId][] = (int)$rootLinePage['uid'];
                    }
                }
            } else {
                if ($GLOBALS['TSFE']->rootLine && is_array($GLOBALS['TSFE']->rootLine)) {
                    foreach ($GLOBALS['TSFE']->rootLine as $rootLinePage) {
                        self::$rootLineIds[$pageId][] = (int)$rootLinePage['uid'];
                    }
                }
            }
        }

        return self::$rootLineIds[$pageId];
    }
}